@@ -131,7 +131,8 @@ class AidlabPeripheral():
                     self.aidlab_delegate.destroy(aidlab_address)
                     self.connected_aidlab.remove(aidlab_address)
                     break
-        except:
+        except Exception as e:
+            logging.info("Exception " + str(e))
             if aidlab_address in self.connected_aidlab: self.connected_aidlab.remove(aidlab_address)
 
 
