@@ -20,13 +20,13 @@ class AidlabSDK:
     samples_time_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, POINTER(c_float), c_int)
     activity_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, c_uint8)
     respiration_rate_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, c_uint32)
-    accelerometer_callback_type = CFUNCTYPE(None, c_void_p, c_float, c_float, c_float, c_uint64)
-    gyroscope_callback_type = CFUNCTYPE(None, c_void_p, c_float, c_float, c_float, c_uint64)
-    magnetometer_callback_type = CFUNCTYPE(None, c_void_p, c_float, c_float, c_float, c_uint64)
+    accelerometer_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, c_float, c_float, c_float)
+    gyroscope_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, c_float, c_float, c_float)
+    magnetometer_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, c_float, c_float, c_float)
     battery_callback_type = CFUNCTYPE(None, c_void_p, c_uint8)
     steps_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, c_uint64)
-    orientation_callback_type = CFUNCTYPE(None, c_void_p, c_float, c_float, c_float, c_uint64)
-    quaternion_callback_type = CFUNCTYPE(None, c_void_p, c_float, c_float, c_float, c_float, c_uint64)
+    orientation_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, c_float, c_float, c_float, c_uint8)
+    quaternion_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, c_float, c_float, c_float, c_float)
     wear_state_callback_type = CFUNCTYPE(None, c_void_p, c_uint8)
     heart_rate_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, POINTER(c_int), c_int)
     sound_volume_callback_type = CFUNCTYPE(None, c_void_p, c_uint64, c_uint16)
@@ -34,7 +34,8 @@ class AidlabSDK:
     received_command_callback_type = CFUNCTYPE(None, c_void_p)
     sync_state_callback_type = CFUNCTYPE(None, c_void_p, c_uint8)
     unsynchronized_size_callback_type = CFUNCTYPE(None, c_void_p, c_uint8)
-    plank_callback_type = CFUNCTYPE(None, c_void_p)
+    received_message_type = CFUNCTYPE(None, c_void_p, POINTER(c_char), POINTER(c_char))
+    user_event_callback_type = CFUNCTYPE(None, c_void_p)
 
     Activity_type = {
         1: "automotive",
@@ -58,7 +59,9 @@ class AidlabSDK:
         2: "sitUp",
         3: "burpee",
         4: "pullUp",
-        5: "squat"
+        5: "squat",
+        6: "plankStart",
+        7: "plankEnd" 
     }
 
     Sync_state = {
@@ -68,11 +71,18 @@ class AidlabSDK:
         3: "empty"
     }
 
+    Body_position = {
+        0: "unknown",
+        1: "front",
+        2: "back",
+        3: "left side",
+        4: "right side"
+    }
+
     ecgFiltrationMethod = {"normal": False, "aggressive": True}
 
     def __init__(self, delegate, aidlab_address):
         self.delegate = delegate
-
         self.aidlab = IAidlab(self)
         self.aidlab.address = aidlab_address
 
@@ -90,13 +100,10 @@ class AidlabSDK:
             raise RuntimeError("Unsupported operating system: {}".format(sys.platform))
 
         self.lib.initial.restype = POINTER(AidlabSDK_ptr)
-
         self.aidlab_sdk_ptr = self.lib.initial()
         # setting up type of variables and return values
         self.setup_process_types()
 
-        
-
     def calculate_temperature(self, data):
         self.lib.processTemperaturePackage((c_uint8 * len(data))(*data), len(data), self.aidlab_sdk_ptr)
 
@@ -190,16 +197,16 @@ class AidlabSDK:
         self.sound_volume_c_callback = self.sound_volume_callback_type(self.sound_volume_callback)
         self.exercise_c_callback = self.exercise_callback_type(self.exercise_callback)
         self.receive_command_c_callback = self.received_command_callback_type(self.did_receive_command)
-        self.plank_start_c_callback = self.plank_callback_type(self.did_start_plank_callback)
-        self.plank_end_c_callback = self.plank_callback_type(self.did_end_plank_callback)
+        self.received_message_c_callback = self.received_message_type(self.did_received_message)
+        self.user_event_c_callback = self.user_event_callback_type(self.user_event_callback)
 
         self.lib.initUserServiceCallback(self.ecg_c_callback, self.respiration_c_callback, self.temperature_c_callback,
                                          self.accelerometer_c_callback, self.gyroscope_c_callback, self.magnetometer_c_callback,
                                          self.battery_c_callback, self.activity_c_callback, self.steps_c_callback, 
                                          self.orientation_c_callback, self.quaternion_c_callback, self.respiration_rate_c_callback,
                                          self.wear_state_c_callback, self.heart_rate_c_callback, self.sound_volume_c_callback,
-                                         self.exercise_c_callback, self.receive_command_c_callback, self.plank_start_c_callback, 
-                                         self.plank_end_c_callback, self.aidlab_sdk_ptr)
+                                         self.exercise_c_callback, self.receive_command_c_callback, self.received_message_c_callback,
+                                         self.user_event_c_callback, self.aidlab_sdk_ptr)
 
     def setup_synchronization_callback(self):
 
@@ -263,7 +270,8 @@ class AidlabSDK:
                                                      self.battery_callback_type, self.activity_callback_type, self.steps_callback_type,
                                                      self.orientation_callback_type, self.quaternion_callback_type, self.respiration_rate_callback_type,
                                                      self.wear_state_callback_type, self.heart_rate_callback_type, self.sound_volume_callback_type,
-                                                     self.exercise_callback_type, self.received_command_callback_type, c_void_p]
+                                                     self.exercise_callback_type, self.received_command_callback_type, self.received_message_type,
+                                                     self.user_event_callback_type, c_void_p]
         self.lib.initUserServiceCallback.restype = None
 
         self.lib.destroy.argtypes = [c_void_p]
@@ -272,15 +280,12 @@ class AidlabSDK:
         self.lib.get_command.argtypes = [POINTER(c_uint8), c_void_p]
         self.lib.get_command.restype = POINTER(c_uint8)
 
+    def user_event_callback(self, context):
+        self.delegate.did_detect_user_event(self.aidlab)
+
     def exercise_callback(self, context, exercise):
         exercise = self.Exercise.get(exercise, "None")
         self.delegate.did_detect_exercise(self.aidlab, exercise)
-
-    def did_start_plank_callback(self, context):
-        self.delegate.did_start_plank()
-
-    def did_end_plank_callback(self, context):
-        self.delegate.did_end_plank()
         
     def ecg_callback(self, context, timestamp, values, size):
         values = [values[i] for i in range(size)]
@@ -296,19 +301,20 @@ class AidlabSDK:
     def temperature_callback(self, context, timestamp, value):
         self.delegate.did_receive_skin_temperature(self.aidlab, timestamp, value)
 
-    def accelerometer_callback(self, context, ax, ay, az, timestamp):
+    def accelerometer_callback(self, context, timestamp, ax, ay, az):
         self.delegate.did_receive_accelerometer(self.aidlab, timestamp, ax, ay, az)
 
-    def gyroscope_callback(self, context, gx, gy, gz, timestamp):
+    def gyroscope_callback(self, context, timestamp, gx, gy, gz):
         self.delegate.did_receive_gyroscope(self.aidlab, timestamp, gx, gy, gz)
 
-    def magnetometer_callback(self, context, mx, my, mz, timestamp):
+    def magnetometer_callback(self, context, timestamp, mx, my, mz):
         self.delegate.did_receive_magnetometer(self.aidlab, timestamp, mx, my, mz)
 
-    def orientation_callback(self, context, roll, pitch, yaw, timestamp):
+    def orientation_callback(self, context, timestamp, roll, pitch, yaw, body_position):
+        body_position = self.Body_position.get(body_position, "unknown")
         self.delegate.did_receive_orientation(self.aidlab, timestamp, roll, pitch, yaw)
 
-    def quaternion_callback(self, context, qw, qx, qy, qz, timestamp):
+    def quaternion_callback(self, context, timestamp, qw, qx, qy, qz):
         self.delegate.did_receive_quaternion(self.aidlab, timestamp, qw, qx, qy, qz)
 
     def activity_callback(self, context, timestamp, activity):
@@ -334,9 +340,11 @@ class AidlabSDK:
 
     def did_receive_command(self, context):
         self.delegate.did_receive_command(self.aidlab)
+    
+    def did_received_message(self, context, process, message):
+        pass
 
     # Synchronization
-
     def sync_state_did_change(self, context, sync_state):
         sync_state = self.Sync_state.get(sync_state, "empty")
         self.delegate.sync_state_did_change(self.aidlab, sync_state)
@@ -366,4 +374,4 @@ class AidlabSDK:
         self.delegate.did_receive_past_activity(self.aidlab, timestamp, activity)
 
     def did_receive_past_steps(self, context, timestamp, value):
-        self.delegate.did_receive_past_steps(self.aidlab, timestamp, value)
+        self.delegate.did_receive_past_steps(self.aidlab, timestamp, value)
\ No newline at end of file
